/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import QtQuick.LocalStorage 2.0
import "../common/ClockUtils.js" as Utils

QtObject {
    id: alarmStorage

    property var db: null

    Component.onCompleted: {
        Utils.log("AlarmStorage loaded")
        openDB();
    }

    // Opens the local db if not already open. On the first call creates the tables
    function openDB () {
        if(db !== null) return;

        db = LocalStorage.openDatabaseSync("ubuntu-clock-app", "0.1", "Default Ubuntu touch clock", 1000);

        try {
            db.transaction(function(tx){
                tx.executeSql('CREATE TABLE IF NOT EXISTS Alarm(key INTEGER PRIMARY KEY, hou INTEGER, min INTEGER, mon INTEGER, tue INTEGER, wed INTEGER, thu INTEGER, fri INTEGER, sat INTEGER, sun INTEGER, ton INTEGER, vib INTEGER, act INTEGER)');
            });
        } catch (err) {
            Utils.error("Error creating table in db");
        }
    }

    //Read alarm from database
    function getAlarmValue () {
        openDB();

        var result = new Array();
        try {
            db.readTransaction(
                        function(tx){
                            var res = tx.executeSql('SELECT * FROM Alarm ORDER BY hou, min');
                            if (res.rows.length > 0) {
                                for (var i = 0; i < res.rows.length; i++) {
                                    result[i] = new Array();
                                    result[i][0] = res.rows.item(i).key;
                                    result[i][1] = res.rows.item(i).hou;
                                    result[i][2] = res.rows.item(i).min;
                                    result[i][3] = res.rows.item(i).mon;
                                    result[i][4] = res.rows.item(i).tue;
                                    result[i][5] = res.rows.item(i).wed;
                                    result[i][6] = res.rows.item(i).thu;
                                    result[i][7] = res.rows.item(i).fri;
                                    result[i][8] = res.rows.item(i).sat;
                                    result[i][9] = res.rows.item(i).sun;
                                    result[i][10] = res.rows.item(i).ton;
                                    result[i][11] = res.rows.item(i).vib;
                                    result[i][12] = res.rows.item(i).act;
                                }
                            }
                        });
        } catch (err) {
            Utils.error("Error getting timer: " + err);
        }
        return result;
    }

    //Read single alarm from database
    function getSingleAlarmValue (alarmkey) {
        openDB();

        var result = new Array();
        try {
            db.readTransaction(
                        function(tx){
                            var res = tx.executeSql('SELECT * FROM Alarm WHERE key = ? ORDER BY hou, min', [alarmkey]);
                            if (res.rows.length > 0) {
                                result[0] = new Array();
                                result[0][0] = res.rows.item(0).key;
                                result[0][1] = res.rows.item(0).hou;
                                result[0][2] = res.rows.item(0).min;
                                result[0][3] = res.rows.item(0).mon;
                                result[0][4] = res.rows.item(0).tue;
                                result[0][5] = res.rows.item(0).wed;
                                result[0][6] = res.rows.item(0).thu;
                                result[0][7] = res.rows.item(0).fri;
                                result[0][8] = res.rows.item(0).sat;
                                result[0][9] = res.rows.item(0).sun;
                                result[0][10] = res.rows.item(0).ton;
                                result[0][11] = res.rows.item(0).vib;
                                result[0][12] = res.rows.item(0).act;
                            }
                        });
        } catch (err) {
            Utils.error("Error getting single timer: " + err);
        }
        return result;
    }

    //Write alarm to database
    function setAlarmValue ( hou, min, mon, tue, wed, thu, fri, sat, sun, ton, vib ) {
        openDB();

        try {
            db.transaction(function(tx){
                var res = tx.executeSql('INSERT INTO Alarm VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)', [null, hou, min, mon, tue, wed, thu, fri, sat, sun, ton, vib, 1]);
            });
        } catch (err) {
            Utils.error("Error setting alarm " + hou + "." + min + ": " + err);
        }
    }

    //Update alarm to database
    function updateAlarmValue ( key, hou, min, mon, tue, wed, thu, fri, sat, sun, ton, vib ) {
        openDB();

        try {
            db.transaction(function(tx){
                var res = tx.executeSql('UPDATE Alarm SET hou = ?, min = ?, mon = ?, tue = ?, wed = ?, thu = ?, fri = ?, sat = ?, sun = ?, ton = ?, vib = ? WHERE key = ?', [hou, min, mon, tue, wed, thu, fri, sat, sun, ton, vib, key]);
            });
        } catch (err) {
            Utils.error("Error update alarm " + key + ": " + err);
        }
    }

    //Enabled/Disabled alarm
    function setStateAlarmValue ( keyalarm , alarmstate ) {
        openDB();
        try {
            db.transaction(function(tx){
                var res = tx.executeSql('UPDATE Alarm SET act = ? WHERE key = ?', [alarmstate, keyalarm]);
            });
        } catch (err) {
            Utils.error("Error change state alarm " + keyalarm + ": " + err);
        }
    }

    // Delete table
    function deleteTable ( table ) {
        openDB();

        try {
            db.transaction(
                        function(tx){
                            tx.executeSql("DROP TABLE " + table);
                        });
        } catch (err) {
            Utils.error("Error delete table '" + table + "': " + err);
        }
    }

    //Delete alarm on database
    function delAlarmValue ( key ) {
        openDB();

        try {
            db.transaction(
                        function(tx){
                            tx.executeSql('DELETE FROM Alarm WHERE key = ?', [key]);
                        });
        } catch (err) {
            Utils.error("Error delete Alarm: " + err);
        }
    }

    //Read alarm set from database
    function getActiveAlarm () {
        openDB();

        var today = new Date();
        var hoursnow = Qt.formatDateTime(new Date(), "hh");
        var minutesnow = Qt.formatDateTime(new Date(), "mm");
        var days = new Array("sun", "mon", "tue", "wed", "thu", "fri", "sat");
        var result = new Array();
        var numrowsafter;
        var numrowsbefore;
        try {
            db.readTransaction(
                        function(tx){
                            var resafter = tx.executeSql('SELECT * FROM Alarm WHERE ((mon = ? AND tue = ? AND wed = ? AND thu = ? AND fri = ? AND sat = ? AND sun = ?) OR ' + days[today.getDay()] + ' = ?) AND act = ? AND ((hou > ' + hoursnow + ') OR (hou = ' + hoursnow + ' AND min > ' + minutesnow + ')) ORDER BY hou, min', [0, 0, 0, 0, 0, 0, 0, 1, 1]);
                            numrowsafter = (resafter.rows.length > 0) ? resafter.rows.length : 0;
                            if (resafter.rows.length > 0) {
                                for (var i = 0; i < resafter.rows.length; i++) {
                                    result[i] = new Array();
                                    result[i][0] = resafter.rows.item(i).key;
                                    result[i][1] = resafter.rows.item(i).hou;
                                    result[i][2] = resafter.rows.item(i).min;
                                    result[i][3] = resafter.rows.item(i).ton;
                                    result[i][4] = resafter.rows.item(i).vib;
                                    result[i][5] = resafter.rows.item(i).act;
                                    result[i][6] = resafter.rows.item(i).mon;
                                    result[i][7] = resafter.rows.item(i).tue;
                                    result[i][8] = resafter.rows.item(i).wed;
                                    result[i][9] = resafter.rows.item(i).thu;
                                    result[i][10] = resafter.rows.item(i).fri;
                                    result[i][11] = resafter.rows.item(i).sat;
                                    result[i][12] = resafter.rows.item(i).sun;
                                    result[i][13] = 0;
                                }
                            }
                        });
        } catch (err) {
            Utils.error("Error getting alarm rep: " + err);
        }
        try {
            db.readTransaction(
                        function(tx){
                            var resbefore = tx.executeSql('SELECT * FROM Alarm WHERE ((mon = ? AND tue = ? AND wed = ? AND thu = ? AND fri = ? AND sat = ? AND sun = ?) OR ' + days[today.getDay()] + ' = ?) AND act = ? AND ((hou < ' + hoursnow + ') OR (hou = ' + hoursnow + ' AND min < ' + minutesnow + ')) ORDER BY hou, min', [0, 0, 0, 0, 0, 0, 0, 1, 1]);
                            numrowsbefore = (resbefore.rows.length > 0) ? resbefore.rows.length : 0;
                            if (resbefore.rows.length > 0) {
                                for (var x = numrowsafter; x < resbefore.rows.length; x++) {
                                    result[x] = new Array();
                                    result[x][0] = resbefore.rows.item(x).key;
                                    result[x][1] = resbefore.rows.item(x).hou;
                                    result[x][2] = resbefore.rows.item(x).min;
                                    result[x][3] = resbefore.rows.item(x).ton;
                                    result[x][4] = resbefore.rows.item(x).vib;
                                    result[x][5] = resbefore.rows.item(x).act;
                                    result[x][6] = resbefore.rows.item(x).mon;
                                    result[x][7] = resbefore.rows.item(x).tue;
                                    result[x][8] = resbefore.rows.item(x).wed;
                                    result[x][9] = resbefore.rows.item(x).thu;
                                    result[x][10] = resbefore.rows.item(x).fri;
                                    result[x][11] = resbefore.rows.item(x).sat;
                                    result[x][12] = resbefore.rows.item(x).sun;
                                    result[x][13] = 0;
                                }
                            }
                        });
        } catch (err) {
            Utils.error("Error getting alarm rep: " + err);
        }
        if (numrowsafter > 0 || numrowsbefore > 0) {
            return result;
        } else {
            return getNextWeekAlarm();
        }
    }

    //if there isn't an alarm in current day read alarms in the next dayset from database
    function getNextWeekAlarm () {
        openDB();

        var today = new Date();
        var chkday = today.getDay() + 1;
        chkday = (chkday == 7) ? 0 : chkday;
        var days = new Array("sun", "mon", "tue", "wed", "thu", "fri", "sat");
        var numdays = 0;
        var result;
        while (chkday != today.getDay()) {
            try {
                db.readTransaction(
                            function(tx){
                                result = new Array();
                                var resnextweek = tx.executeSql('SELECT * FROM Alarm WHERE ' + days[chkday] + ' = ? AND act = ? ORDER BY hou, min', [1, 1]);
                                if (resnextweek.rows.length > 0) {
                                    for (var i = 0; i < resnextweek.rows.length; i++) {
                                        result[i] = new Array();
                                        result[i][0] = resnextweek.rows.item(i).key;
                                        result[i][1] = resnextweek.rows.item(i).hou;
                                        result[i][2] = resnextweek.rows.item(i).min;
                                        result[i][3] = resnextweek.rows.item(i).ton;
                                        result[i][4] = resnextweek.rows.item(i).vib;
                                        result[i][5] = resnextweek.rows.item(i).act;
                                        result[i][6] = resnextweek.rows.item(i).mon;
                                        result[i][7] = resnextweek.rows.item(i).tue;
                                        result[i][8] = resnextweek.rows.item(i).wed;
                                        result[i][9] = resnextweek.rows.item(i).thu;
                                        result[i][10] = resnextweek.rows.item(i).fri;
                                        result[i][11] = resnextweek.rows.item(i).sat;
                                        result[i][12] = resnextweek.rows.item(i).sun;
                                        result[i][13] = 24 * numdays;
                                    }
                                    chkday = today.getDay();
                                } else {
                                    chkday++;
                                    numdays++;
                                    chkday = (chkday == 7) ? 0 : chkday;
                                }
                            });
            } catch (err) {
                Utils.error("Error getting alarm rep2: " + err);
            }
        }
        return result;
    }
}
