/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Renato Araujo Oliveira Filho <renato.filho@canonical.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common/ClockUtils.js" as Utils
import "StopwatchSupport.js" as SScript
import "../common"

Page {
    id: stopwatchPage

    flickable: null

    property alias lapStart: laps.lapStart

    Component.onCompleted: {
        Utils.log("StopwatchPage loaded");
    }

    // Timer which runs every 100 microseconds to update the stopwatch label
    Timer {
        id: stopWatchLabelUpdate;

        interval: 100;
        repeat: true;
        onTriggered: labelStopwatch.text = SScript.intervalToString(analogStopwatch.hours, analogStopwatch.minutes, analogStopwatch.seconds, analogStopwatch.milliseconds)
    }

    ListModel {
        id: laps
        objectName: "laps"

        property int lapStart: 0;

        function push() {
            insert(0, {"time" : (analogStopwatch.totalTime - lapStart),
                       "label": SScript.currentLapLabel(analogStopwatch.totalTime, lapStart)})
            lapStart = analogStopwatch.totalTime
        }
    }

    // Items within this container will move up/down depending on the state.
    AnimationContainer {
        id: stopwatchAnimationContainer

        // Label to show the current time
        Item {
            id: labelContainer;

            z: analogStopwatch.z + 1;
            width: analogStopwatch.innerCircle.width; height: width;
            anchors.centerIn: analogStopwatch;

            Label {
                id: labelStopwatch
                objectName: "labelStopwatch"

                anchors.centerIn: parent
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                color: Theme.palette.normal.baseText

                fontSize: "x-large"
                text: labelStopwatch.text = SScript.intervalToString(analogStopwatch.hours, analogStopwatch.minutes, analogStopwatch.seconds, analogStopwatch.milliseconds)
            }
        }

        // Component which draws the analog stopwatch face, controls the timer and calculates the time.
        AnalogStopwatch {
            id: analogStopwatch
            objectName: "buttonStopwatch"

            anchors { top: parent.top; topMargin: units.gu(9); horizontalCenter: parent.horizontalCenter }

            MouseArea {
                anchors.fill: analogStopwatch.innerCircle;
                onClicked: {
                    if(analogStopwatch.timerStatus == false) {
                        analogStopwatch.start()
                        stopWatchLabelUpdate.start()
                        if (analogStopwatch.flag) {
                            analogStopwatch.startTime = SScript.timeToMs(new Date());
                            analogStopwatch.flag = false;
                        }
                        else {
                            analogStopwatch.delay = Math.abs(SScript.timeToMs(new Date()) - analogStopwatch.stopTime);
                            analogStopwatch.startTime = analogStopwatch.startTime + analogStopwatch.delay;
                        }
                    } else {
                        analogStopwatch.stop()
                        stopWatchLabelUpdate.stop()
                        analogStopwatch.stopTime = SScript.timeToMs(new Date());
                    }
                }
            }
        }

        ImageButton {
            id: resetButton
            objectName: "resetButton"

            anchors { top: analogStopwatch.bottom; topMargin: units.gu(4); left: parent.left; leftMargin: units.gu(4) }
            buttonImage: Qt.resolvedUrl("../images/reset_icon.png");
            buttonLabel: i18n.tr("Reset");

            onClicked: {
                analogStopwatch.stop()
                analogStopwatch.reset();
                labelStopwatch.text = SScript.intervalToString(0, 0, 0, 0);
                lapStart = 0;
                laps.clear();
            }
        }

        ImageButton {
            id: lapButton
            objectName: "lapButton"

            anchors { top: analogStopwatch.bottom; topMargin: units.gu(4); right: parent.right; rightMargin: units.gu(4) }
            buttonImage: Qt.resolvedUrl("../images/lap_icon.png");
            buttonLabel: i18n.tr("Lap");
            mirrorLayout: true;

            enabled: analogStopwatch.timerStatus == true ? true : false;

            onClicked: {
                laps.push();
                analogStopwatch.blipLocation = analogStopwatch.seconds
                analogStopwatch.blipVisible = true
            }
        }

        // ListView to display the current lap and recorded lap information.
        Column {
            id: listLap

            height: childrenRect.height;
            anchors { left: parent.left; right: parent.right; top: analogStopwatch.bottom; topMargin: units.gu(13) }

            ListItem.ThinDivider {}

            ListItem.Header {
                Label {
                    text: i18n.tr("Laps")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListItem.Standard {
                Label {
                    text: i18n.tr("Current")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                    color: Theme.palette.normal.baseText
                    fontSize: "large"
                }

                Label {
                    text: SScript.currentLapLabel(analogStopwatch.totalTime, laps.lapStart);
                    anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "large"
                }
            }

            ListItem.Header {
                Label {
                    text: i18n.tr("Recorded laps")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListView {
                id: listViewLap

                clip: true
                anchors { left: parent.left; right: parent.right; }
                height: units.gu(21)
                model: laps

                delegate: ListItem.Standard {
                    Label {
                        text: Utils.numberToLocaleString(laps.count - index);
                        anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }

                    Label {
                        text: label;
                        anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }
                }
            }
        }
    }
}
