# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
#
# Copyright (C) 2013 Canonical Ltd
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 3 as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Authored by: Riccardo Padovani <rpadovani@ubuntu-it.org>

"""Tests for the Clock App - Timer"""

from __future__ import absolute_import

from testtools.matchers import Equals, NotEquals
from autopilot.matchers import Eventually
from time import sleep
from autopilot.platform import model
from ubuntu_clock_app.tests import ClockAppTestCase

class TestTimer(ClockAppTestCase):
    """Tests the timer page features"""

    """ This is needed to wait for the application to start.
        In the testfarm, the application may take some time to show up."""
    def setUp(self):
        super(TestTimer, self).setUp()
        self.assertThat(self.main_window.get_qml_view().visible, Eventually(Equals(True)))

        self.move_to_timer_tab()

        # Test if the current Tab is the Timer Tab
        CurrentTab = self.main_window.get_object("Tab", "TimerTab")
        self.assertThat(CurrentTab.title, Eventually(Equals("Timer")))


    def drag_page_up_to_show_input_box(self):
        timer_page = self.app.select_single("TimerPage")

        x, y, w, h = timer_page.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2)

        self.pointing_device.drag(tx, ty, tx, ty - h / 3)
        self.assertThat(
            timer_page.select_single("AnimationContainer").moving,
            Eventually(Equals(False)))

    def drag_page_down_to_previous_state(self):
        timer_page = self.app.select_single("TimerPage")

        x, y, w, h = timer_page.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2)

        self.pointing_device.drag(tx, ty, tx, ty + h / 3)
        self.assertThat(
            timer_page.select_single("AnimationContainer").moving,
            Eventually(Equals(False)))


    def open_toolbar(self):
        toolbar = self.get_toolbar()

        x, y, w, h = toolbar.globalRect
        tx = x + (w / 2)
        ty = y + (h - (h / 8))

        self.pointing_device.drag(tx, ty, tx, ty - h)
        self.assertThat(toolbar.animating, Eventually(Equals(False)))
        #self.assertThat(toolbar.state, Eventually(Equals("spread")))

    def close_toolbar(self):
        toolbar = self.get_toolbar()

        x, y, w, h = toolbar.globalRect
        tx = x + (w / 2)
        ty = y + (h - (h / 8))

        self.pointing_device.drag(tx, ty - h, tx, ty)
        self.assertThat(toolbar.animating, Eventually(Equals(False)))
        #self.assertThat(toolbar.state, Eventually(NotEquals("spread")))

    def test_add_preset(self):
        """Test to check if button to add a preset working"""

        # Show the toolbar
        self.open_toolbar()

        # Click the add preset button
        add_preset_button = self.main_window.get_toolbar_timer_add_preset_button()
        self.pointing_device.click_object(add_preset_button)

        #For now close the toolbar due to https://bugs.launchpad.net/ubuntu-clock-app/+bug/1209024
        self.close_toolbar()

        # Write in the label
        if model() != "Desktop":
            self.drag_page_up_to_show_input_box()
        label = self.main_window.get_timer_name_preset()
        self.assertThat(lambda: label, Eventually(NotEquals(None)))
        self.pointing_device.click_object(label)
        self.assertThat(label.focus, Eventually(Equals(True)))
        self.keyboard.type("test")
        self.assertThat(label.text, Eventually(Equals("test")))

        if model() != "Desktop":
            self.drag_page_down_to_previous_state()

        # Set hour
        hour = self.main_window.get_timer_hour_hand()
        self.assertThat(lambda: hour, Eventually(NotEquals(None)))
        x, y, w, h = hour.globalRect
        tx = x + (w / 2)
        ty = y + (h / 2)
        self.pointing_device.drag(tx, ty - (h / 4), tx + (w / 2), ty + (h / 2))

        num_of_presets_old = int(self.main_window.get_num_of_presets())
        # Press "Done" button
        label = self.main_window.get_timer_clock_center_label()
        self.pointing_device.click_object(label)

        # Check if the preset has been saved
        self.assertThat(self.main_window.get_num_of_presets(),
            Eventually(Equals(num_of_presets_old+1)))

    def test_delete_preset(self):
        """Test if the swipe of a preset deletes it"""

        # Create a new preset
        self.test_add_preset()

        num_of_presets_old = int(self.main_window.get_num_of_presets())
        first_preset = self.main_window.get_first_preset_list_item()
        self.assertThat(lambda: first_preset, Eventually(NotEquals(None)))

        if model() != "Desktop":
            self.drag_page_up_to_show_input_box()
        # Delete the preset
        x, y, w, h = first_preset.globalRect
        tx = x + (w / 8)
        ty = y + (h / 2)
        self.pointing_device.drag(tx, ty, w - w / 8, ty)

        # Check has been deleted
        self.assertThat(self.main_window.get_num_of_presets(),
            Eventually(Equals(num_of_presets_old-1)))

    def test_run_preset(self):
        """Tests if a preset can runs"""

        # Create a new preset
        self.test_add_preset()

        # Click to select the preset
        first_preset = self.main_window.get_first_preset_list_item()
        self.assertThat(lambda: first_preset, Eventually(NotEquals(None)))
        if model() != "Desktop":
            self.drag_page_up_to_show_input_box()
        self.pointing_device.click_object(first_preset)

        if model() != "Desktop":
            self.drag_page_down_to_previous_state()

        # Click to start the preset
        label = self.main_window.get_timer_clock_center_label()
        self.pointing_device.click_object(label)

        # Check if timer is started
        label_timer = self.main_window.get_label_timer()
        self.assertThat(label_timer.text, Eventually(NotEquals("23:00:00")))
