/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Renato Araujo Oliveira Filho <renato@canonical.com>
 */

import QtQuick 2.0
import QtQuick.LocalStorage 2.0
import "../common/ClockUtils.js" as Utils

ListModel {
    id: model

    readonly property string dbName: "ubuntu-clock-app"
    readonly property string dbVersion: "0.1"
    readonly property string dbDescription: "Preset timers"

     property var _db: null

    Component.onCompleted: {
        _loadDB()
    }

    function appendPreset(label, seconds)
    {
        if (_appendDB(label, seconds)) {
            model.append({"label" : label, "seconds" : seconds})
        }
    }

    function removePreset(index)
    {
        var label = get(index).label
        if (_removeDB(label)) {
            model.remove(index);
        }
    }

    function _createDB() {
        if (_db == null) return false;
        try {
            _db.transaction(function(tx){
                tx.executeSql('CREATE TABLE IF NOT EXISTS Timer(label TEXT, seconds INTEGER)');
            });
        } catch (err) {
            Utils.error("Error creating table in db:" + err);
            return false;
        }
        return true;
    }

    function _loadDB() {
        _db = LocalStorage.openDatabaseSync(model.dbName, model.dbVersion, model.dbDescription, 1000);
        if (_db == null) return false;
        if (!_createDB()) {
            _db = null;
            return false;
        }
        try {
            _db.readTransaction(function(tx){
                var res = tx.executeSql('SELECT * FROM Timer');
                if (res.rows.length > 0) {
                    for (var i = 0; i < res.rows.length; i++) {
                        model.append({"label" : res.rows.item(i).label,
                                      "seconds" : res.rows.item(i).seconds});
                    }
                }
            });
        } catch (err) {
            Utils.error("Error opening database: " + err);
            _db = null
            return false;
        }
        return true;
    }

    function _appendDB(label, seconds) {
        if (_db == null) return false;

        try {
            _db.transaction(function(tx){
                var currentValue;
                var res = tx.executeSql('SELECT seconds FROM Timer WHERE label=?', label);
                if (res.rows.length > 0) {
                    currentValue = res.rows.item(0).seconds;
                }

                if (currentValue !== seconds) {
                    // Update existing value or insert if none
                    if (currentValue !== undefined) {
                        res = tx.executeSql('UPDATE Timer SET seconds = ? WHERE label = ?', [seconds, label]);
                    } else {
                        res = tx.executeSql('INSERT INTO Timer VALUES(?, ?)', [label, seconds]);
                    }
                }
            });
        } catch (err) {
            Utils.error("Error setting labeltxt '"+ label + "': " + err);
            return false;
        }
        return true;
    }

    function _removeDB(label) {
        if (_db == null) return false;

        try {
            _db.transaction(function(tx){
                tx.executeSql('DELETE FROM Timer WHERE label = ?', [label]);
            });
        } catch (err) {
            Utils.error("Error delete Timer'" + label + "': " + err);
            return false;
        }
        return true;
    }
}
