/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Alessandro Pozzi <signor.hyde@gmail.com>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import "../common"
import "../common/ClockUtils.js" as Utils

Page {
    id: timerPage

    flickable: null

    property alias hours: analogTimer.hours;
    property alias minutes: analogTimer.minutes;
    property alias seconds: analogTimer.seconds;
    property alias totalTime: analogTimer.totalTime;
    property alias timerOn: analogTimer.timerOn;

    function onTimerUpdate() {
        analogTimer.onTimerUpdate();
    }

    function reset() {
        analogTimer.reset();
        namePreset.text = "";
    }

    // Function to return total seconds.
    function totalSeconds(hou, min, sec) {
        return( (hou * 60 * 60) + (min * 60) + sec );
    }

    // Function to print timer label (format mm:ss and hh:mm:ss when hh > 0) at top
    function intervalToString(hours, minutes, seconds)
    {
        var locale = Qt.locale();

        if (hours > 0) {
            return Utils.zeroleft(hours, 2) + ":" + Utils.zeroleft(minutes, 2) + ":" + Utils.zeroleft(seconds, 2);
        } else {
            return Utils.zeroleft(minutes, 2) + ":" + Utils.zeroleft(seconds, 2);
        }
    }

    // Function to calculate and print the hours, minutes and seconds from the total time as a string
    function getstringTimer(time) {
        var hours, minutes, seconds;

        seconds = time % 60
        time = Math.floor(time / 60)
        minutes = time % 60
        hours = Math.floor(time / 60)

        return Utils.hmsToString(hours, minutes, seconds);
    }

    Component.onCompleted: {
        Utils.log("TimerPage loaded");
    }

    PresetTimersModel {
        id: presetModel
    }

    states: [
        State {
            name: "ADDPRESET"
            PropertyChanges { target: addPresetCircleDone; visible: true }
            PropertyChanges { target: namePreset; visible: true }
        }
    ]

    AnimationContainer {
        id: timerAnimationContainer

        // Label to show the current time
        Item {
            id: labelContainer;

            z: analogTimer.z + 1;
            width: analogTimer.innerCircle.width; height: width;
            anchors.centerIn: analogTimer;

            Label {
                id: labelTimer
                objectName: "labelTimer"

                anchors.centerIn: parent
                horizontalAlignment: Text.AlignHCenter
                verticalAlignment: Text.AlignVCenter
                color: Theme.palette.normal.baseText

                fontSize: "x-large"
                text: labelTimer.text = intervalToString(hours, minutes, seconds);
            }
        }

        // Main element which draws the timer face and manages time.
        AnalogTimer {
            id: analogTimer

            anchors { top: parent.top; topMargin: units.gu(9); horizontalCenter: parent.horizontalCenter }
            inProgressFlag: (timerOn == true || analogTimer.state == "DONE") ? true : false;
            onSecondsChanged: labelTimer.text = intervalToString(hours, minutes, seconds);
            onMinutesChanged: labelTimer.text = intervalToString(hours, minutes, seconds);
            onHoursChanged: labelTimer.text = intervalToString(hours, minutes, seconds);

            states:[
                State { name: "STOP" },
                State { name: "PAUSE"},
                State {
                    name: "DONE"
                    when: (totalTime == -1 && timerOn == false)
                    PropertyChanges { target: analogTimer; showGlow: true }
                    PropertyChanges { target: analogTimer; glowColor: "#03CB4A" }
                }
            ]

            MouseArea {
                anchors.fill: analogTimer.innerCircle
                onClicked: {
                    switch (analogTimer.state) {

                    case "":
                        if ((seconds != 0 || hours != 0 || minutes != 0) || timerOn) {
                            totalTime = totalSeconds(hours, minutes, seconds);
                            analogTimer.startTime = new Date();
                            timerOn = true;
                            analogTimer.state = "STOP";
                        }

                        if (timerAnimationContainer.state == "SCROLLED") {
                            timerAnimationContainer.swipeView()
                        }
                        break;

                    case "STOP":
                        timerOn = false;
                        analogTimer.pauseTime = analogTimer.totalTime - analogTimer.remTime + 1; // +1 is necessary to start immediately the timer when finish a pause
                        analogTimer.state = "PAUSE";
                        break;

                    case "DONE":
                        reset();
                        analogTimer.state = "";
                        break;

                    case "PAUSE":
                        analogTimer.startTime = new Date();
                        timerOn = true;
                        analogTimer.state = "STOP";
                        break;
                    }
                }
            }
        }

        ImageButton {
            id: resetButtonTimer
            objectName: "resetButtonTimer"

            anchors { top: analogTimer.bottom; topMargin: units.gu(4); horizontalCenter: analogTimer.horizontalCenter; leftMargin: units.gu(4) }
            buttonImage: Qt.resolvedUrl("../images/reset_icon.png");
            buttonLabel: i18n.tr("Reset");

            enabled: true

            onClicked: {
                reset();
                analogTimer.state = "";
            }
        }

        // Column element to hold the saved presets
        Column {
            id: listPreset

            height: childrenRect.height
            visible: timerPage.state == "ADDPRESET" ? false : true;
            anchors { left: parent.left; right: parent.right; top: analogTimer.bottom; topMargin: units.gu(13) }

            ListItem.ThinDivider { }

            ListItem.Header {
                Label {
                    text: i18n.tr("Presets")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListView {
                id: listTimerPreset

                anchors { left: parent.left; right: parent.right }
                height: units.gu(21)
                model: presetModel
                currentIndex: -1

                delegate: ListItem.Standard {
                    objectName: "preset" + index
                    Label {
                        id: presetTextLabel // Needs by autopilot
                        objectName: "presetTextLabel" + index
                        fontSize: "large";
                        text: label
                        anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                        color: Theme.palette.normal.baseText
                    }

                    Label {
                        objectName: "presetTimerLabel"
                        fontSize: "large"
                        text: getstringTimer(seconds);
                        anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                        color: Theme.palette.normal.baseText
                    }

                    selected: listTimerPreset.currentIndex == index
                    removable: true

                    backgroundIndicator: SwipeDelete {
                        id: swipeDelete
                        state: swipingState
                    }

                    // TODO: Implement different based actions based on the swipe direction.
                    onItemRemoved: {
                        presetModel.removePreset(index);
                    }

                    onClicked: {
                        if (!timerOn) {
                            analogTimer.ssToTime(seconds);
                            listTimerPreset.currentIndex = index
                        }
                    }
                }
            }
        }

        // Add preset done circle which is shown when the add preset toolbar button is pressed
        Rectangle {
            id: addPresetCircleDone

            width: units.gu(12); height: units.gu(12)
            radius: width / 2
            anchors.centerIn: analogTimer
            color: "transparent"
            antialiasing: true
            visible: false

            Label {
                id: addPresetTextDone
                objectName: "addPresetTextDone"

                //anchors.centerIn: parent
                color: "lightgreen"
                fontSize: "x-large"

                text: i18n.tr("DONE")
            }

            MouseArea {
                anchors.fill: parent
                onClicked: {
                    if (namePreset.text != "" && (hours > 0 || minutes > 0 || seconds > 0)) {
                        presetModel.appendPreset(namePreset.text, totalSeconds(hours, minutes, seconds));
                        reset();
                        timerPage.state = ""
                    }
                }
            }
        }

        // Element to set the name of a saved preset.
        TextField {
            id: namePreset
            objectName: "namePreset"

            visible: false
            hasClearButton: true
            anchors { left: analogTimer.left; right: analogTimer.right; top: analogTimer.bottom; topMargin: units.gu(10)}

            placeholderText: i18n.tr("Timer One")
        }
    }

    // Various toolbar actions such as add preset, edit and back.
    tools: ToolbarItems {
        id: toolbarTimer

        ToolbarButton {
            action: Action {
                id: addPreset

                iconSource: Qt.resolvedUrl("../images/add_icon.png")
                text: i18n.tr("Add Preset")
                onTriggered: {
                    reset()
                    if (timerAnimationContainer.state == "SCROLLED") {
                        timerAnimationContainer.swipeView()
                    }
                    timerPage.state = "ADDPRESET"
                }
            }
            visible: timerPage.state == "ADDPRESET" ? false : true;
        }

        ToolbarButton {
            action: Action {
                id: deletePreset

                iconSource: Qt.resolvedUrl("../images/remove_icon.png")
                text: i18n.tr("Delete")
                onTriggered: {
                    if (!timerOn) {
                        presetModel.removePreset(listTimerPreset.currentIndex);
                        analogTimer.reset();
                    }
                }
            }
            visible: timerAnimationContainer.state == "SCROLLED" ? true : false;
        }

        back: ToolbarButton {
            text: "Back";
            iconSource: Qt.resolvedUrl("../images/back_icon.png");
            visible: timerPage.state == "ADDPRESET" ? true : false;

            onTriggered: {
                if (timerAnimationContainer.state == "SCROLLED") {
                    timerAnimationContainer.swipeView()
                }
                else if (timerPage.state == "ADDPRESET") {
                    timerPage.state = ""
                    reset()
                }
            }
        }
    }
}
